(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
	typeof define === 'function' && define.amd ? define(factory) :
	(global.H5Player = factory());
}(this, (function () { 'use strict';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

var RE_LYRIC = /^\[([\d:.]+)\]\s*(.*)$/;

function getTime(str) {
  var time = 0;
  str.split(':').forEach(function (part) {
    time = time * 60 + +part;
  });
  return time;
}

var LyricParser =
/*#__PURE__*/
function () {
  function LyricParser() {
    this.reset();
  }

  var _proto = LyricParser.prototype;

  _proto.reset = function reset() {
    this.data = [];
    this.index = 0;
  };

  _proto.setLyric = function setLyric(lyric) {
    this.reset();
    var data = this.data;
    (lyric || '').split('\n').forEach(function (line) {
      var matches = line.match(RE_LYRIC);
      if (matches) data.push([getTime(matches[1]), matches[2]]);
    });
  };

  _proto.getLyricByTime = function getLyricByTime(time) {
    var data = this.data;
    var index = this.index;
    var last = data[index] || data[index = 0];

    if (last) {
      var step = last[0] > time ? -1 : 1;

      while (true) {
        // eslint-disable-line no-constant-condition
        var item = data[index];
        var next = data[index + 1];
        if ((!item || item[0] <= time) && (!next || next[0] > time)) break;
        index += step;
      }
    }

    var current = data[this.index = index];
    return current ? current[1] : '';
  };

  return LyricParser;
}();

var EventEmitter =
/*#__PURE__*/
function () {
  function EventEmitter() {
    Object.defineProperty(this, "map", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: {}
    });
  }

  var _proto = EventEmitter.prototype;

  _proto.on = function on(type, handle) {
    var _this = this;

    var handlers = this.map[type];

    if (!handlers) {
      handlers = [];
      this.map[type] = handlers;
    }

    handlers.push(handle);
    return function () {
      return _this.off(type, handle);
    };
  };

  _proto.off = function off(type, handle) {
    var handlers = this.map[type];

    if (handlers) {
      var i = handlers.indexOf(handle);
      if (i >= 0) handlers.splice(i, 1);
    }
  };

  _proto.once = function once(type, handle) {
    var revoke = this.on(type, handleOnce);
    return revoke;

    function handleOnce() {
      handle.apply(void 0, arguments);
      revoke();
    }
  };

  _proto.emit = function emit(type) {
    for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
      args[_key - 1] = arguments[_key];
    }

    var handlers = this.map[type];

    if (handlers) {
      handlers.forEach(function (handle) {
        handle.apply(void 0, args);
      });
    }
  };

  return EventEmitter;
}();

function prevent(e) {
  if (e && e.preventDefault) {
    e.preventDefault();
    e.stopPropagation();
  }
}
function createElement(tagName, props, children) {
  var el = document.createElement(tagName);

  if (props) {
    Object.keys(props).forEach(function (key) {
      var value = props[key];

      if (key === 'on') {
        bindEvents(el, value);
      } else {
        el[key] = value;
      }
    });
  }

  if (children) {
    children.forEach(function (child) {
      el.appendChild(child);
    });
  }

  return el;
}
function bindEvents(el, events) {
  if (events) {
    Object.keys(events).forEach(function (type) {
      var handle = events[type];
      if (handle) el.addEventListener(type, handle);
    });
  }

  return el;
}
function empty(el) {
  el.innerHTML = '';
  return el;
}
var NS_SVG = 'http://www.w3.org/2000/svg';
var NS_XLINK = 'http://www.w3.org/1999/xlink';
function createSVGElement(tagName, children) {
  var el = document.createElementNS(NS_SVG, tagName);

  if (children) {
    children.forEach(function (child) {
      el.appendChild(child);
    });
  }

  return el;
}
function createSVGIcon(name) {
  var use = createSVGElement('use');
  use.setAttributeNS(NS_XLINK, 'href', "#" + name);
  return createSVGElement('svg', [use]);
}

var Progress =
/*#__PURE__*/
function (_EventEmitter) {
  _inheritsLoose(Progress, _EventEmitter);

  function Progress() {
    var _this;

    _this = _EventEmitter.call(this) || this;
    Object.defineProperty(_assertThisInitialized(_this), "handleCursor", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(e) {
        prevent(e);
        _this.cursorData = {
          delta: e.clientX - _this.els.played.offsetWidth
        };
        document.addEventListener('mousemove', _this.handleCursorMove, false);
        document.addEventListener('mouseup', _this.handleCursorEnd, false);
      }
    });
    Object.defineProperty(_assertThisInitialized(_this), "handleCursorMove", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(e) {
        prevent(e);
        _this.cursorData.moved = true;

        _this.setCursorPos(_this.getPos(e));
      }
    });
    Object.defineProperty(_assertThisInitialized(_this), "handleCursorChange", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(e) {
        var x;

        if ('offsetX' in e) {
          x = e.offsetX;
        } else {
          var rect = e.target.getBoundingClientRect();
          var docEl = document.documentElement;
          var win = window;
          x = e.pageX - (rect.left + win.pageXOffset - docEl.clientLeft);
        }

        var pos = x / _this.els.bar.offsetWidth;

        _this.setCursorPos(pos);

        _this.emit('cursor', pos);
      }
    });
    Object.defineProperty(_assertThisInitialized(_this), "handleCursorEnd", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(e) {
        document.removeEventListener('mousemove', _this.handleCursorMove, false);
        document.removeEventListener('mouseup', _this.handleCursorEnd, false);

        var pos = _this.getPos(e);

        _this.cursorData = null;

        _this.setCursorPos(pos);

        _this.emit('cursor', pos);
      }
    });

    _this.build();

    return _this;
  }

  var _proto = Progress.prototype;

  _proto.build = function build() {
    var played = createElement('div', {
      className: 'h5p-played'
    });
    var bar = createElement('div', {
      className: 'h5p-bar'
    }, [played]);
    var cursor = createElement('div', {
      className: 'h5p-cursor',
      on: {
        mousedown: this.handleCursor,
        click: prevent
      }
    });
    var time = createElement('div', {
      className: 'h5p-time'
    });
    var el = createElement('div', {
      className: 'h5p-progress',
      on: {
        click: this.handleCursorChange
      }
    }, [bar, cursor, time]);
    this.el = el;
    this.els = {
      bar: bar,
      played: played,
      cursor: cursor,
      time: time
    };
  };

  _proto.setCursor = function setCursor(currentTime, duration) {
    if (!this.cursorData) this.setCursorPos(duration ? currentTime / duration : null);
    this.els.time.textContent = formatTime(currentTime) + " / " + formatTime(duration);
  };

  _proto.setCursorPos = function setCursorPos(pos) {
    var _els = this.els,
        played = _els.played,
        cursor = _els.cursor;
    var past = (pos || 0) * 100 + "%";
    played.style.width = past;
    cursor.style.left = past;
  };

  _proto.getPos = function getPos(e) {
    var pos = (e.clientX - this.cursorData.delta) / this.els.bar.offsetWidth;
    return Math.max(0, Math.min(1, pos));
  };

  return Progress;
}(EventEmitter);

function formatTime(time) {
  var minutes = time / 60 | 0;
  var seconds = time % 60 | 0;
  return leftpadNumber(minutes, 2) + ":" + leftpadNumber(seconds, 2);
}

function leftpadNumber(num, len) {
  var pad = Number.isNaN(num) ? '?' : '0';
  var str;

  for (str = "" + num; str.length < len; str = "" + pad + str) {
  }

  return str;
}

var svgSprite = "<svg xmlns=\"http://www.w3.org/2000/svg\">\n<symbol id=\"h5p-backward\" viewBox=\"0 0 1024 1024\"><path d=\"M778.857 80.571q10.857-10.857 18.286-7.428t7.428 18.286V932.57q0 14.858-7.428 18.286t-18.286-7.428L373.143 537.714q-5.143-5.143-7.429-10.857v387.429q0 14.857-10.857 25.714t-25.714 10.857H256q-14.857 0-25.714-10.857t-10.857-25.714V109.714q0-14.857 10.857-25.714T256 73.143h73.143Q344 73.143 354.857 84t10.857 25.714v387.429q2.286-6.286 7.429-10.857z\"/></symbol>\n<symbol id=\"h5p-forward\" viewBox=\"0 0 1024 1024\"><path d=\"M245.143 943.429q-10.857 10.857-18.286 7.428t-7.428-18.286V91.43q0-14.858 7.428-18.286t18.286 7.428l405.714 405.715q4.572 4.571 7.429 10.857v-387.43q0-14.857 10.857-25.714t25.714-10.857H768q14.857 0 25.714 10.857t10.857 25.714v804.572q0 14.857-10.857 25.714T768 950.857h-73.143q-14.857 0-25.714-10.857t-10.857-25.714V526.857q-2.857 5.714-7.429 10.857z\"/></symbol>\n<symbol id=\"h5p-list\" viewBox=\"0 0 1024 1024\"><path d=\"M219.429 804.571q0 45.715-32 77.715t-77.715 32-77.714-32T0 804.57t32-77.714 77.714-32 77.715 32 32 77.714zm0-292.571q0 45.714-32 77.714t-77.715 32-77.714-32T0 512t32-77.714 77.714-32 77.715 32 32 77.714zM1024 749.714V859.43q0 7.428-5.429 12.857t-12.857 5.428H310.857q-7.428 0-12.857-5.428t-5.429-12.857V749.714q0-7.428 5.429-12.857t12.857-5.428h694.857q7.429 0 12.857 5.428t5.429 12.857zM219.429 219.43q0 45.714-32 77.714t-77.715 32-77.714-32T0 219.43t32-77.715 77.714-32 77.715 32 32 77.715zM1024 457.143v109.714q0 7.429-5.429 12.857t-12.857 5.429H310.857q-7.428 0-12.857-5.429t-5.429-12.857V457.143q0-7.429 5.429-12.857t12.857-5.429h694.857q7.429 0 12.857 5.429t5.429 12.857zm0-292.572v109.715q0 7.428-5.429 12.857t-12.857 5.428H310.857q-7.428 0-12.857-5.428t-5.429-12.857V164.57q0-7.428 5.429-12.857t12.857-5.428h694.857q7.429 0 12.857 5.428T1024 164.57z\"/></symbol>\n<symbol id=\"h5p-pause\" viewBox=\"0 0 1024 1024\"><path d=\"M950.857 109.714v804.572q0 14.857-10.857 25.714t-25.714 10.857H621.714q-14.857 0-25.714-10.857t-10.857-25.714V109.714Q585.143 94.857 596 84t25.714-10.857h292.572Q929.143 73.143 940 84t10.857 25.714zm-512 0v804.572q0 14.857-10.857 25.714t-25.714 10.857H109.714Q94.857 950.857 84 940t-10.857-25.714V109.714Q73.143 94.857 84 84t25.714-10.857h292.572Q417.143 73.143 428 84t10.857 25.714z\"/></symbol>\n<symbol id=\"h5p-play\" viewBox=\"0 0 1024 1024\"><path d=\"M900.571 529.714L141.714 951.43q-13.143 7.428-22.571 1.714t-9.429-20.572V91.43q0-14.858 9.429-20.572t22.571 1.714l758.857 421.715q13.143 7.428 13.143 17.714t-13.143 17.714z\"/></symbol>\n<symbol id=\"h5p-repeat-off\" viewBox=\"0 0 1024 1024\"><path d=\"M85.333 224.853l54.614-54.186 713.386 713.386-54.186 54.614-128-128h-372.48v128L128 768l170.667-170.667v128h287.146L298.667 438.187v31.146h-85.334v-116.48l-128-128m640 329.814h85.334v178.346l-85.334-85.333v-93.013m0-341.334v-128L896 256 725.333 426.667v-128H376.32l-85.333-85.334h434.346z\"/></symbol>\n<symbol id=\"h5p-repeat-one\" viewBox=\"0 0 1024 1024\"><path d=\"M554.667 640V384H512l-85.333 42.667v42.666h64V640m234.666 85.333H298.667v-128L128 768l170.667 170.667v-128h512v-256h-85.334m-426.666-256h426.666v128L896 256 725.333 85.333v128h-512v256h85.334V298.667z\"/></symbol>\n<symbol id=\"h5p-repeat\" viewBox=\"0 0 1024 1024\"><path d=\"M725.333 725.333H298.667v-128L128 768l170.667 170.667v-128h512v-256h-85.334m-426.666-256h426.666v128L896 256 725.333 85.333v128h-512v256h85.334V298.667z\"/></symbol>\n</svg>";

function initialize() {
  var _document = document,
      body = _document.body;

  if (!body) {
    document.addEventListener('DOMContentLoaded', initialize);
    return;
  }

  var sprite = createElement('div', {
    innerHTML: svgSprite
  });
  sprite.style.display = 'none';
  body.insertBefore(sprite, body.firstChild);
}

initialize();

var H5P_ACTIVE = 'h5p-active';
var MODES = ['repeatAll', 'repeatOne', 'repeatOff'];
var MODE_ICONS = {
  repeatAll: 'h5p-repeat',
  repeatOne: 'h5p-repeat-one',
  repeatOff: 'h5p-repeat-off'
}; // manage all the players to ensure only one is playing at once

var players = [];
var currentPlayer = null;

function setCurrentPlayer(player) {
  currentPlayer = player;
  players.forEach(function (other) {
    if (player !== other) other.audio.pause();
  });
}

function fireEvent(detail) {
  var event = new CustomEvent('PlayerEvent', {
    detail: detail,
    bubbles: true,
    cancelable: true
  });
  document.dispatchEvent(event);
}

var Player =
/*#__PURE__*/
function () {
  function Player(options) {
    var _this = this;

    Object.defineProperty(this, "handleSwitchMode", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(e) {
        prevent(e);
        var index = MODES.indexOf(_this.mode);

        _this.setMode(MODES[(index + 1) % MODES.length]);
      }
    });
    Object.defineProperty(this, "handleToggleList", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(e) {
        prevent(e);

        _this.setPlaylist(!_this.els.buttons.list.classList.contains(H5P_ACTIVE));
      }
    });
    Object.defineProperty(this, "handleTogglePlay", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(e) {
        prevent(e);
        if (_this.current < 0) _this.play(0);else if (_this.audio.paused) _this.audio.play();else _this.audio.pause();
      }
    });
    Object.defineProperty(this, "handlePlayPrev", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(e) {
        prevent(e);

        _this.play(_this.prev());
      }
    });
    Object.defineProperty(this, "handlePlayNext", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(e) {
        prevent(e);

        _this.play(_this.next());
      }
    });
    Object.defineProperty(this, "handlePlayAnother", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value() {
        var mode = _this.mode;

        if (mode === 'repeatAll') {
          _this.handlePlayNext();
        } else if (mode === 'repeatOne') {
          _this.play();
        } else {
          var next = _this.next();

          if (next) _this.play(next);
        }
      }
    });
    Object.defineProperty(this, "handleUpdateTime", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(e) {
        var target = e.target;
        var currentTime = target.currentTime;
        _this.duration = target.duration || _this.duration;

        _this.progress.setCursor(currentTime, _this.duration);

        _this.els.lyric.textContent = _this.lyricParser.getLyricByTime(currentTime);
      }
    });
    Object.defineProperty(this, "handleStatusChange", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(e) {
        var type = e.type;
        var isPlaying = type === 'play';

        if (isPlaying) {
          setCurrentPlayer(_this);
          fireEvent({
            type: type,
            player: _this
          });
        } else if (currentPlayer === _this) {
          currentPlayer = null;
          fireEvent({
            type: type,
            player: _this
          });
        }

        var play = _this.els.buttons.play;
        play.firstChild.replaceWith(createSVGIcon(isPlaying ? 'h5p-pause' : 'h5p-play'));

        _this.els.image.classList.toggle('h5p-roll', isPlaying);
      }
    });
    Object.defineProperty(this, "handlePlayItem", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(e) {
        prevent(e);
        var childNodes = _this.els.playlist.childNodes;

        for (var i = 0; i < childNodes.length; i += 1) {
          var child = childNodes[i];

          if (child === e.target) {
            _this.play(i);

            break;
          }
        }
      }
    });
    Object.defineProperty(this, "handleCursorChange", {
      configurable: true,
      enumerable: true,
      writable: true,
      value: function value(pos) {
        var currentTime = _this.duration * pos | 0;
        _this.audio.currentTime = currentTime;

        _this.play();
      }
    });
    players.push(this);
    this.build(options);
    this.setSongs([]);
    this.setTheme(options.theme);
    this.setMode(options.mode);
    this.setPlaylist(options.showPlaylist);
  }

  var _proto = Player.prototype;

  _proto.build = function build(options) {
    this.defaultImage = options.image || '';
    this.callbackGetLyric = options.getLyric;
    this.progress = new Progress();
    var buttons = {};
    var image = createElement('div', {
      className: 'h5p-image'
    });
    var toolbar = createElement('div', {
      className: 'h5p-toolbar'
    }, [buttons.repeat = createElement('i', {
      className: 'h5p-button',
      on: {
        click: this.handleSwitchMode
      }
    }, [createSVGIcon('h5p-repeat')]), buttons.list = createElement('i', {
      className: 'h5p-button',
      on: {
        click: this.handleToggleList
      }
    }, [createSVGIcon('h5p-list')])]);
    var title = createElement('div', {
      className: 'h5p-title'
    });
    var artist = createElement('div', {
      className: 'h5p-artist'
    });
    var info = createElement('div', {
      className: 'h5p-info'
    }, [title, artist]);
    var control = createElement('div', {
      className: 'h5p-control'
    }, [createElement('i', {
      className: 'h5p-button',
      on: {
        click: this.handlePlayPrev
      }
    }, [createSVGIcon('h5p-backward')]), buttons.play = createElement('i', {
      className: 'h5p-button',
      on: {
        click: this.handleTogglePlay
      }
    }, [createSVGIcon('h5p-play')]), createElement('i', {
      className: 'h5p-button',
      on: {
        click: this.handlePlayNext
      }
    }, [createSVGIcon('h5p-forward')])]);
    var progress = createElement('div', {
      className: 'h5p-progress-wrap'
    }, [this.progress.el]);
    var lyric = createElement('div', {
      className: 'h5p-lyric'
    });
    var playlist = createElement('div', {
      className: 'h5p-playlist',
      on: {
        click: this.handlePlayItem
      }
    });
    var audio = bindEvents(new Audio(), {
      ended: this.handlePlayAnother,
      timeupdate: this.handleUpdateTime,
      play: this.handleStatusChange,
      pause: this.handleStatusChange
    });
    this.progress.on('cursor', this.handleCursorChange);
    this.audio = audio;
    this.el = createElement('div', {
      className: 'h5p'
    }, [image, toolbar, info, control, progress, lyric, playlist, audio]);
    this.els = {
      image: image,
      buttons: buttons,
      lyric: lyric,
      playlist: playlist,
      title: title,
      artist: artist
    };
    this.lyricParser = new LyricParser();
  };

  _proto.destroy = function destroy() {
    var el = this.el;
    var parent = el.parentNode;
    if (parent) parent.removeChild(el);
  };

  _proto.play = function play(index) {
    if (index == null) index = this.current;
    var song = this.songs[index];
    if (!song) song = this.songs[index = 0];

    if (song) {
      if (this.current !== index) {
        var childNodes = this.els.playlist.childNodes;
        var last = childNodes[this.current];
        if (last) last.classList.remove(H5P_ACTIVE);
        this.current = index;
        childNodes[index].classList.add(H5P_ACTIVE);
        this.audio.src = song.url;
        this.duration = song.duration ? song.duration / 1000 : null;
        this.showInfo(song);
        this.progress.setCursor(0, this.duration);
      }

      this.audio.play();
    }
  };

  _proto.prev = function prev() {
    return (this.current + this.songs.length - 1) % this.songs.length;
  };

  _proto.next = function next() {
    return (this.current + 1) % this.songs.length;
  };

  _proto.setSongs = function setSongs(songs) {
    this.songs = songs;
    var playlist = this.els.playlist;
    empty(playlist);
    songs.forEach(function (_ref) {
      var name = _ref.name;
      playlist.appendChild(createElement('div', {
        title: name,
        textContent: name
      }));
    });
    this.current = -1;
    this.audio.src = '';
    this.duration = 0;
    this.showInfo(this.songs[0]);
  };

  _proto.showInfo = function showInfo(song) {
    this.updateInfo(song);

    var _ref2 = song || {},
        name = _ref2.name,
        artist = _ref2.artist;

    var els = this.els;
    els.title.textContent = name || '';
    els.artist.textContent = artist || '';
  };

  _proto.updateInfo = function updateInfo(item) {
    var song = item || this.songs[this.current];

    var _ref3 = song || {},
        image = _ref3.image;

    if (typeof image === 'object') image = image[this.theme];
    image = image || this.defaultImage;
    var els = this.els;
    var imageEl = empty(els.image);

    if (image) {
      imageEl.appendChild(createElement('img', {
        src: image
      }));
    }

    els.lyric.textContent = '';
    if (song) this.loadLyric(song);
  };

  _proto.loadLyric = function loadLyric(song) {
    var _this2 = this;

    var lyricParser = this.lyricParser;

    if (song.lyric == null) {
      lyricParser.setLyric();
      var callbackGetLyric = this.callbackGetLyric;

      if (callbackGetLyric) {
        callbackGetLyric(_extends({}, song), function (lyric) {
          if (song === _this2.songs[_this2.current]) {
            lyricParser.setLyric(song.lyric = lyric || '');
          }
        });
      }
    } else {
      lyricParser.setLyric(song.lyric);
    }
  };

  _proto.setTheme = function setTheme(name) {
    var themes = Player.themes;
    var index = themes.indexOf(name);
    if (index < 0) index = 0;
    var oldTheme = this.theme;
    this.theme = themes[index];

    if (oldTheme !== this.theme) {
      var classList = this.el.classList;
      classList.remove("h5p-" + oldTheme);
      classList.add("h5p-" + this.theme);
      this.updateInfo();
    }
  };

  _proto.setMode = function setMode(mode) {
    this.mode = MODES.indexOf(mode) < 0 ? MODES[0] : mode;
    var icon = MODE_ICONS[this.mode];
    this.els.buttons.repeat.firstChild.replaceWith(createSVGIcon(icon));
  };

  _proto.setPlaylist = function setPlaylist(show) {
    var _els = this.els,
        playlist = _els.playlist,
        buttons = _els.buttons;
    buttons.list.classList.toggle(H5P_ACTIVE, !!show);
    playlist.style.display = show ? 'block' : '';
  };

  return Player;
}();

Object.defineProperty(Player, "themes", {
  configurable: true,
  enumerable: true,
  writable: true,
  value: ['normal', 'simple']
});

return Player;

})));
